/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.Fg009Default
 *
 * de.bsvrz.kex.tls.osi7.conversion.Fg009Default is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg009Default is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.Fg009Default.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.Fg009Default.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg009Default ist Freie Software: Sie können es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 *  veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg009Default wird in der Hoffnung, dass es nützlich sein wird, aber
 *  OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License für weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */



package de.bsvrz.kex.tls.osi7.conversion;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.kex.tls.osi7.*;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.hexdump.HexDumper;

import java.util.ArrayList;
import java.util.List;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Umsetungsmodul für die FG 9, Standard (alle Typen der FG 9 gemäß TLS 2011) Wird standardmäßig für alle FG 9 De
 * verwendet, bei denen keine spezielle Umsezungsklasse in der Konfiguration angegeben wurde.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class Fg009Default extends FgAllDefault {

    /**
     * DebugLogger für Debug-Ausgaben
     */
    private static final Debug debug = Debug.getLogger();

    //~ FELDER ================================================================

    // Private ----------------------------------------------------------------

    /**
     * Tabelle, in der das Verhalten beim Empfang von Attributgruppen eingetragen wird. Jede Tabellenzeile hat folgenden
     * Aufbau:
     * <pre>
     * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
     * z.B.
     * "atg.tlsGloKanalSteuerung", "asp.parameterSoll", "9", "29", "2", "typ.deZfr"
     * </pre>
     * Die Auswertung erfolgt dabei wie folgt:
     * <ul>
     * <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine Empfangsanmeldung als Empfänger.</li>
     * <li> In der Methode {@link #update} wird für jede zur Attributgruppen/Aspket-Kombination passende Kombination
     * eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag wie folgt gebildet wird (hier für den
     * Beispieleintrag):<br> <code>Fg009Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll(ResultData)</code><br>
     * Dabei enthält <code>ResultData</code> die Daten der zu verarbeitenden ATG/ASP-Kombination.</li>
     * <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine Empfangsanmeldung als Empfänger. Sind mehrere
     * passende Eintrage vorhanden, werden entsprechend mehrere Methodenaufrufe durchgeführt. Dadurch können mehrere
     * ATG/ASP-Kombinationen in einen De-Block abgebildet werden (z.B. protokollnahe TLS-Attributgruppe und
     * Attributgruppe eines Fachmodells):</li>
     * <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </li>
     * </ul>
     */
    private final Object[][] davToTls = new Object[][] {

        // Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
        { "atg.tlsGloDeBlockAbruf", "asp.tlsAbruf", "255", "255", "255", "typ.deZfr" },                      // x
        { "atg.tlsGloDeBlockSenden", "asp.tlsVorgabe", "255", "255", "255", "typ.deZfr" },                   // x
        { "atg.tlsGloKanalSteuerung", "asp.parameterSoll", "255", "29", "2", "typ.deZfr" },                  // x
        { "atg.tlsGloGeographischeKenndaten", "asp.tlsVorgabe", "255", "36", "3", "typ.deZfr" },             // x
        { "atg.tlsZfrBetriebsArt", "asp.parameterSoll", "9", "17", "2", "typ.deZfr" },                       // x
        { "atg.tlsZfrAbrufPufferInhalt", "asp.tlsAbruf", "9", "20", "2", "typ.deZfr" },                      // x
        { "atg.tlsZfrBetriebsParameterZuflussRegelung", "asp.parameterSoll", "9", "32", "3", "typ.deZfr" },  // x
        { "atg.tlsZfrBetriebsParameterAlinea1", "asp.parameterSoll", "9", "33", "3", "typ.deZfr" },          // x
        { "atg.tlsZfrBetriebsParameterRotFahrerZähler", "asp.parameterSoll", "9", "34", "3", "typ.deZfr" },  // x
        { "atg.tlsZfrBetriebsParameterAlinea2", "asp.parameterSoll", "9", "35", "3", "typ.deZfr" },          // x
        { "atg.tlsZfrSignalPlan", "asp.tlsVorgabe", "9", "48", "5", "typ.deZfr" },                           // x
        { "atg.tlsZfrHelligkeit", "asp.tlsVorgabe", "9", "49", "5", "typ.deZfr" },                           // x
    };

    /**
     * Tabelle, in der das Verhalten beim Empfang von TLS De-Blöcken eingetragen wird. Jede Tabellenzeile hat folgenden
     * Aufbau:
     * <pre>
     * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
     * z.B.
     * "atg.tlsGloKanalSteuerung", "asp.tlsAntwort", "9", "29", "2", "typ.deZfr"
     * </pre>
     * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine
     * Sendeanmeldung als Quelle.</li> <li> In der Methode {@link #processDeBlock} wird für jede zum De-Block passende
     * Fg/Id/Typ-Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag wie folgt gebildet wird (hier für
     * den Beispieleintrag):<br> <code>Fg009Typ029Id002AtgTlsGloKanalSteuerungAspTlsAntwort(DeBlock, Data,
     * DataDescription)</code><br> Dabei enthält der <code>deBlock</code> die Daten des zu verarbeitenden De-Blocks und
     * <code>data</code> ein Datenobjekt der passenden Attributgruppen/Aspekt-Kombination, welchem direkt die Inhalte des
     * De-Blocks zugewiesen werden können.<code>DataDescription</code> enthält die passende {@link
     * de.bsvrz.dav.daf.main.DataDescription} für die zu sendenden Daten.</li> <li> Für jede Attributguppen/Aspekt
     * Kombination erfolgt eine Sendeanmeldung als Quelle. Sind mehrere passende Eintrage vorhanden, werden entsprechend
     * mehrere Methodenaufrufe durchgeführt. Dadurch kann ein De-Block mehrfach ausgewert und an verschiedene
     * Attributgruppen weitergerleitet werden (z.B. protokollnahe TLS-Attributgruppe und Attributgruppe eine
     * Fachmodells):</li> <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </ul>
     */
    private final Object[][] tlsToDav = new Object[][] {

        // Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
        { "atg.tlsGloDeFehler", "asp.tlsAntwort", "9", "1", "1", "typ.deZfr" },                              // x
        { "atg.tlsGloDeFehler", "asp.tlsAntwort", "9", "1", "33", "typ.deZfr" },                             // x
        { "atg.tlsGloKanalSteuerung", "asp.parameterIst", "255", "29", "2", "typ.deZfr" },                   // x
        { "atg.tlsGloKanalSteuerung", "asp.parameterIst", "255", "29", "34", "typ.deZfr" },                  // x
        { "atg.tlsGloGeographischeKenndaten", "asp.tlsZustand", "255", "36", "3", "typ.deZfr" },             // x
        { "atg.tlsGloGeographischeKenndaten", "asp.tlsZustand", "255", "36", "35", "typ.deZfr" },            // x
        { "atg.tlsZfrErgänzendeDeFehlerMeldung", "asp.tlsAntwort", "9", "14", "1", "typ.deZfr" },            // x
        { "atg.tlsZfrErgänzendeDeFehlerMeldung", "asp.tlsAntwort", "9", "14", "33", "typ.deZfr" },           // x
        { "atg.tlsZfrNegativeQuittung", "asp.tlsAntwort", "9", "16", "2", "typ.deZfr" },                     // x
        { "atg.tlsZfrNegativeQuittung", "asp.tlsAntwort", "9", "16", "34", "typ.deZfr" },                    // x
        { "atg.tlsZfrBetriebsArt", "asp.parameterIst", "9", "17", "2", "typ.deZfr" },                        // x
        { "atg.tlsZfrBetriebsArt", "asp.parameterIst", "9", "17", "34", "typ.deZfr" },                       // x
        { "atg.tlsZfrBetriebsParameterZuflussRegelung", "asp.parameterIst", "9", "32", "3", "typ.deZfr" },   // x
        { "atg.tlsZfrBetriebsParameterZuflussRegelung", "asp.parameterIst", "9", "32", "35", "typ.deZfr" },  // x
        { "atg.tlsZfrBetriebsParameterAlinea1", "asp.parameterIst", "9", "33", "3", "typ.deZfr" },           // x
        { "atg.tlsZfrBetriebsParameterAlinea1", "asp.parameterIst", "9", "33", "35", "typ.deZfr" },          // x
        { "atg.tlsZfrBetriebsParameterRotFahrerZähler", "asp.parameterIst", "9", "34", "3", "typ.deZfr" },   // x
        { "atg.tlsZfrBetriebsParameterRotFahrerZähler", "asp.parameterIst", "9", "34", "35", "typ.deZfr" },  // x
        { "atg.tlsZfrBetriebsParameterAlinea2", "asp.parameterIst", "9", "35", "3", "typ.deZfr" },             // x
        { "atg.tlsZfrBetriebsParameterAlinea2", "asp.parameterIst", "9", "35", "35", "typ.deZfr" },            // x
        { "atg.tlsZfrSignalPlan", "asp.tlsZustand", "9", "48", "5", "typ.deZfr" },                           // x
        { "atg.tlsZfrSignalPlan", "asp.tlsZustand", "9", "48", "37", "typ.deZfr" },                          // x
        { "atg.tlsZfrHelligkeit", "asp.tlsZustand", "9", "49", "5", "typ.deZfr" },                           // x
        { "atg.tlsZfrHelligkeit", "asp.tlsZustand", "9", "49", "37", "typ.deZfr" },                          // x
        { "atg.tlsZfrErgebnisMeldungRotFahrerZähler", "asp.tlsAntwort", "9", "65", "4", "typ.deZfr" },       // x
        { "atg.tlsZfrErgebnisMeldungRotFahrerZähler", "asp.tlsAntwort", "9", "65", "36", "typ.deZfr" },      // x
        { "atg.tlsZfrAlineaSchaltVorschläge", "asp.tlsAntwort", "9", "70", "4", "typ.deZfr" },               // x
        { "atg.tlsZfrAlineaSchaltVorschläge", "asp.tlsAntwort", "9", "70", "36", "typ.deZfr" },              // x
    };

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt ein Objekt vom Typ Fg009Default
     *
     * @param daf Datenverteiler
     * @param eak Das übergeordnete EAK dieses DE
     * @param de  Das DE
     */
    public Fg009Default(ClientDavInterface daf, Eak eak, De de) {
        super(daf, eak, de);
        subscribeAllData(tlsToDav, davToTls);
    }

    //~ METHODEN ==============================================================

    /**
     * FG 9 : Typ: 1 Id: 1 Name: DE-Fehler (Antwort). Beschreibung siehe [TLS 2011].
     * <p>
     * Ergänzt globale Behandlung De-Fehler in {@link #Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort}.
     * <p>
     * Beim Empfang einer De-Gutmeldung wird die DE initialisiert, indem folgende Telegramme verschickt werden:
     * <ul>
     * <li>Typ 17: ZfrBetriebsArt</li>
     * <li>Typ 29: ZfrKanalsteuerung</li>
     * <li>Typ 32: ZfrBetriebsParameterZuflussRegelung</li>
     * <li>Typ 33: ZfrBetriebsParameterAlinea1</li>
     * <li>Typ 34: ZfrBetriebsParameterRotFahrerZähler</li>
     * <li>Typ 35: ZfrBetriebsParameterAlinea2</li>
     * </ul>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData        resultData     = super.Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(deBlock, data, dataDescription);
        IntegerValueState DEFehlerStatus = data.getUnscaledValue("DEFehlerStatus").getState();

        if (DEFehlerStatus != null) {
            debug.fine("DEFehlerStatus: " + DEFehlerStatus.getName());

            if (DEFehlerStatus.getName().equals("ok")) {
                sendInitialDeBlock(deBlock.getKey(), 17, 2);  // ZfrBetriebsart senden
                sendInitialDeBlock(deBlock.getKey(), 29, 2);  // ZfrKanalsteuerung senden
                sendInitialDeBlock(deBlock.getKey(), 32, 3);  // ZfrBetriebsParameterZuflussRegelung senden
                sendInitialDeBlock(deBlock.getKey(), 33, 3);  // ZfrBetriebsParameterAlinea1 senden
                sendInitialDeBlock(deBlock.getKey(), 34, 3);  // ZfrBetriebsParameterRotFahrerZähler senden
                sendInitialDeBlock(deBlock.getKey(), 35, 3);  // ZfrBetriebsParameterAlinea2 senden
            } else {
                List<De> deList = new ArrayList<De>();

                deList.add(_de);
                TlsApplicationLayer.writeNoDataToUnreachableDe(deList, _eakConversionModul.getTimeStamp(), -1, "atg.tlsGloDeFehler");
            }
        }

        return resultData;
    }

    /**
     * FG 9 : Typ: 1 Id: 33 Name: DE-Fehler (Antwort). Standardverarbeitung wie bei Id 1,
     * {@link #Fg009Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(de.bsvrz.kex.tls.osi7.DeBlock, de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)} ()},
     * aber als "nachgelieferte" Daten an den DaV übergeben. Zudem werden bei nachgelieferten Daten keine Sequenzen verschickt.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ001Id033AtgTlsGloDeFehlerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 1, aber:
        // 1. als "nachgelieferte" Daten an den DaV übergeben.
        // 2. bei nachgelieferten Daten werden keine Sequenzen verschickt.
        ResultData resultData = super.Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 9 : Typ: 14 Id: 1  Name: Ergänzende DE-Fehlermeldung (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ014Id001AtgTlsZfrErgänzendeDeFehlerMeldungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Herstellercode-Byte aus Telegramm ---------------------------------------------------------------------------
        int _Herstellercode_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Hersteller").set(_Herstellercode_);

        // Fehler-Bytes aus Telegramm -----------------------------------------------------------------------------------
        deBlock.getDeData().readUnsignedByteAsInt();  // Anzahl der folgenden Bytes überlesen, da immer == 4 !

        int              _TlsFehler1_ = deBlock.getDeData().readUnsignedByteAsInt();
        int              _TlsFehler2_ = deBlock.getDeData().readUnsignedByteAsInt();
        int              _TlsFehler3_ = deBlock.getDeData().readUnsignedByteAsInt();
        int              _TlsFehler4_ = deBlock.getDeData().readUnsignedByteAsInt();
        Data.NumberArray _TlsFehler_  = data.getItem("FehlercodesTLS").getUnscaledArray("Fehler");

        _TlsFehler_.setLength(4);
        _TlsFehler_.getValue(0).set(_TlsFehler1_);
        _TlsFehler_.getValue(1).set(_TlsFehler2_);
        _TlsFehler_.getValue(2).set(_TlsFehler3_);
        _TlsFehler_.getValue(3).set(_TlsFehler4_);

        // Hersteller Fehler-Bytes aus Telegramm -----------------------------------------------------------------------
        int   count              = deBlock.getDeData().readUnsignedByteAsInt();
        int[] _HerstellerFehler_ = new int[count];

        for (int i = 0; i < count; i++) {
            _HerstellerFehler_[i] = deBlock.getDeData().readUnsignedByteAsInt();
        }

        Data.NumberArray _Fehler_ = data.getItem("FehlercodesHersteller").getUnscaledArray("Fehler");

        _Fehler_.setLength(count);

        for (int i = 0; i < count; i++) {
            _Fehler_.getValue(i).set(_HerstellerFehler_[i]);
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 9 : Typ: 14 Id: 33 Name: Ergänzende DE-Fehlermeldung (Antwort). Standardverarbeitung wie bei Id 2,
     * {@link #Fg009Typ014Id001AtgTlsZfrErgänzendeDeFehlerMeldungAspTlsAntwort(de.bsvrz.kex.tls.osi7.DeBlock, de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)},
     * aber als "nachgelieferte" Daten an den DaV übergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ014Id033AtgTlsZfrErgänzendeDeFehlerMeldungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 2, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg009Typ014Id001AtgTlsZfrErgänzendeDeFehlerMeldungAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 9 : Typ: 16 Id: 2 Name: Negative Quittung (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ016Id002AtgTlsZfrNegativeQuittungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        return super.Fg255Typ016Id002AtgTlsGloNegativeQuittungAspTlsAntwort(deBlock, data, dataDescription);
    }

    /**
     * FG 9 : Typ: 16 Id: 34 Name: Negative Quittung (Antwort). Standardverarbeitung wie bei ID 2,
     * {@link #Fg009Typ016Id002AtgTlsZfrNegativeQuittungAspTlsAntwort(de.bsvrz.kex.tls.osi7.DeBlock, de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)},
     * aber als "nachgelieferte" Daten an den DaV übergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ016Id034AtgTlsZfrNegativeQuittungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 2, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg009Typ016Id002AtgTlsZfrNegativeQuittungAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 9 : Typ: 17 Id: 2 Name: Betriebsart (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ017Id002AtgTlsZfrBetriebsArtAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Betriebsart aus Telegramm -----------------------------------------------------------------------
        int _BetriebsArt_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("BetriebsArt").set(_BetriebsArt_);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 9 : Typ: 17 Id: 2 Name: Betriebsart (Senden). Beschreibung siehe [TLS 2011].
     *
     * @param result Der vom Datenverteiler empfangene Datensatz, der in einen De-Block konvertiert wird.
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg009Typ017Id002AtgTlsZfrBetriebsArtAspParameterSoll(ResultData result) {
        DeBlock deBlock = null;
        Data    data    = result.getData();  // Datensatz holen

        // Datenblock beseht aus folgenden Inhalten / Umsetzung
        int _betriebsArt_ = data.getUnscaledValue("BetriebsArt").intValue();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(1);

        deApplicationData.writeIntAsByte(_betriebsArt_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 17, 2, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 9 : Typ: 17 Id: 34 Name: Betriebsart (Antwort). Standardverarbeitung wie bei Id 2,
     * {@link #Fg009Typ017Id002AtgTlsZfrBetriebsArtAspParameterIst(de.bsvrz.kex.tls.osi7.DeBlock, de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)},
     * aber als "nachgelieferte" Daten an den DaV übergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ017Id034AtgTlsZfrBetriebsArtAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 2, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg009Typ017Id002AtgTlsZfrBetriebsArtAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 9 : Typ: 20 Id: 2 Name: Abruf Pufferinhalt (Abruf). Beschreibung siehe [TLS 2011].
     *
     * @param result Der vom Datenverteiler empfangene Datensatz, der in einen De-Block konvertiert wird.
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg009Typ020Id002AtgTlsZfrAbrufPufferInhaltAspTlsAbruf(ResultData result) {
        DeBlock deBlock = null;
        Data    data    = result.getData();  // Datensatz holen

        // Datenblock beseht aus folgenden Inhalten / Umsetzung
        int _startFolgeNummer_ = data.getUnscaledValue("StartFolgeNummer").intValue();
        int _anzahl1_          = data.getUnscaledValue("Anzahl1").intValue();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(3);

        deApplicationData.writeIntAsLowHighByte(_startFolgeNummer_);
        deApplicationData.writeIntAsByte(_anzahl1_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 20, 2, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    // / Typ 32

    /**
     * FG 9 : Typ: 32 Id: 3 Name: Betriebsparameter Zuflussregelung (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ032Id003AtgTlsZfrBetriebsParameterZuflussRegelungAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _gelbZeit_                                       = deBlock.getDeData().readUnsignedByteAsInt();
        int _rotGelbZeit_                                    = deBlock.getDeData().readUnsignedByteAsInt();
        int _mindestDunkelOderGruenZeit_                     = deBlock.getDeData().readUnsignedByteAsInt();
        int _maximaleDunkelOderGruenZeit_                    = deBlock.getDeData().readUnsignedByteAsInt();
        int _maximaleBelegZeit_                              = deBlock.getDeData().readUnsignedByteAsInt();
        int _maximaleDunkelOderGruenZeitBeiDetektorStoerung_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _steuerFlags_                                    = deBlock.getDeData().readUnsignedByteAsInt();

        // GrünFlag und Modus aus den Steuerflags ermitteln
        int _gruenFlag_ = _steuerFlags_ & 1;
        int _modus_     = _steuerFlags_ >> 5 & 7;

        data.getUnscaledValue("GelbZeit").set(_gelbZeit_);
        data.getUnscaledValue("RotGelbZeit").set(_rotGelbZeit_);
        data.getUnscaledValue("MindestDunkelOderGrünZeit").set(_mindestDunkelOderGruenZeit_);
        data.getUnscaledValue("MaximaleDunkelOderGrünZeit").set(_maximaleDunkelOderGruenZeit_);
        data.getUnscaledValue("MaximaleBelegZeit").set(_maximaleBelegZeit_);
        data.getUnscaledValue("MaximaleDunkelOderGrünZeitBeiDetektorStörung").set(_maximaleDunkelOderGruenZeitBeiDetektorStoerung_);
        data.getUnscaledValue("GrünFlag").set(_gruenFlag_);
        data.getUnscaledValue("Modus").set(_modus_);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 9 : Typ: 32 Id: 3 Name: Betriebsparameter Zuflussregelung (Senden). Beschreibung siehe [TLS 2011].
     *
     * @param result Der vom Datenverteiler empfangene Datensatz, der in einen De-Block konvertiert wird.
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg009Typ032Id003AtgTlsZfrBetriebsParameterZuflussRegelungAspParameterSoll(ResultData result) {
        DeBlock deBlock = null;
        Data    data    = result.getData();  // Datensatz holen

        // Datenblock beseht aus folgenden Inhalten / Umsetzung
        int _gelbZeit_                                       = data.getUnscaledValue("GelbZeit").intValue();
        int _rotGelbZeit_                                    = data.getUnscaledValue("RotGelbZeit").intValue();
        int _mindestDunkelOderGruenZeit_                     = data.getUnscaledValue("MindestDunkelOderGrünZeit").intValue();
        int _maximaleDunkelOderGruenZeit_                    = data.getUnscaledValue("MaximaleDunkelOderGrünZeit").intValue();
        int _maximaleBelegZeit_                              = data.getUnscaledValue("MaximaleBelegZeit").intValue();
        int _maximaleDunkelOderGruenZeitBeiDetektorStoerung_ = data.getUnscaledValue("MaximaleDunkelOderGrünZeitBeiDetektorStörung").intValue();
        int _gruenFlag_                                      = data.getUnscaledValue("GrünFlag").intValue();
        int _modus_                                          = data.getUnscaledValue("Modus").intValue();

        // Aus GrünFlag und Modus das Byte Steuerflags1 zusammenbauen
        int _steuerFlags_ = _gruenFlag_ | _modus_ << 5;

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(7);

        deApplicationData.writeIntAsByte(_gelbZeit_);
        deApplicationData.writeIntAsByte(_rotGelbZeit_);
        deApplicationData.writeIntAsByte(_mindestDunkelOderGruenZeit_);
        deApplicationData.writeIntAsByte(_maximaleDunkelOderGruenZeit_);
        deApplicationData.writeIntAsByte(_maximaleBelegZeit_);
        deApplicationData.writeIntAsByte(_maximaleDunkelOderGruenZeitBeiDetektorStoerung_);
        deApplicationData.writeIntAsByte(_steuerFlags_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 32, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 9 : Typ: 32 Id: 35 Name: Betriebsparameter Zuflussregelung (Antwort). Standardverarbeitung wie bei Id 3,
     * {@link #Fg009Typ032Id003AtgTlsZfrBetriebsParameterZuflussRegelungAspParameterIst(de.bsvrz.kex.tls.osi7.DeBlock, de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)},
     * aber als "nachgelieferte" Daten an den DaV übergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ032Id035AtgTlsZfrBetriebsParameterZuflussRegelungAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg009Typ032Id003AtgTlsZfrBetriebsParameterZuflussRegelungAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    // / Typ 33

    /**
     * FG 9 : Typ: 33 Id: 3 Name: Betriebsparameter Alinea1 (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ033Id003AtgTlsZfrBetriebsParameterAlinea1AspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _qEMin_                  = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _korrekturfaktor_        = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _bOpt_                   = deBlock.getDeData().readUnsignedByteAsInt();
        int _berechnungsartBelegung_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _mindesschaltzeit_       = deBlock.getDeData().readUnsignedByteAsInt();
        int _nachlaufzeit_           = deBlock.getDeData().readUnsignedByteAsInt();
        int _maxErsteRotLaufZeit_    = deBlock.getDeData().readUnsignedByteAsInt();
        int _schwaechsteRotLaufZeit_ = deBlock.getDeData().readUnsignedByteAsInt();

        // 3 Reservebytes überlesen
        deBlock.getDeData().readUnsignedByteAsInt();
        deBlock.getDeData().readUnsignedByteAsInt();
        deBlock.getDeData().readUnsignedByteAsInt();

        // Fixen Teil des Telegramms in Ziel-ATG übertragen...
        data.getUnscaledValue("qEMin").set(_qEMin_);
        data.getUnscaledValue("Korrekturfaktor").set(_korrekturfaktor_);
        data.getUnscaledValue("bOpt").set(_bOpt_);
        data.getUnscaledValue("BerechnungsartBelegung").set(_berechnungsartBelegung_);
        data.getUnscaledValue("Mindesschaltzeit").set(_mindesschaltzeit_);
        data.getUnscaledValue("Nachlaufzeit").set(_nachlaufzeit_);
        data.getUnscaledValue("maxErsteRotLaufZeit").set(_maxErsteRotLaufZeit_);
        data.getUnscaledValue("schwächsteRotLaufZeit").set(_schwaechsteRotLaufZeit_);

        // Anzahl der nachfolgenden Programmblöcke auslesen...
        int _anzahlProgramme_ = deBlock.getDeData().readUnsignedByteAsInt();

        // Datenfeld auf passende Länge initialisieren
        data.getArray("PrgAlinea1").setLength(_anzahlProgramme_);

        // ...Schleife über die Programme direkt auslesen und in ATG schreiben
        for (int i = 0; i < _anzahlProgramme_; i++) {

            // Daten aus Telegramm einlesen...
            int _tEin_          = deBlock.getDeData().readUnsignedLowHighByteAsInt();
            int _tAus_          = deBlock.getDeData().readUnsignedLowHighByteAsInt();
            int _rotSchaltZeit_ = deBlock.getDeData().readUnsignedByteAsInt();

            // ...und direkt in ATG / ATL PrgAlinea1 eintragen
            data.getArray("PrgAlinea1").getItem(i).getUnscaledValue("TEin").set(_tEin_);
            data.getArray("PrgAlinea1").getItem(i).getUnscaledValue("TAus").set(_tAus_);
            data.getArray("PrgAlinea1").getItem(i).getUnscaledValue("RotSchaltZeit").set(_rotSchaltZeit_);
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 9 : Typ: 33 Id: 3 Name: Betriebsparameter Alinea1 (Senden). Beschreibung siehe [TLS 2011].
     *
     * @param result Der vom Datenverteiler empfangene Datensatz, der in einen De-Block konvertiert wird.
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg009Typ033Id003AtgTlsZfrBetriebsParameterAlinea1AspParameterSoll(ResultData result) {
        DeBlock deBlock = null;
        Data    data    = result.getData();  // Datensatz holen

        // Datenblock beseht aus folgenden Inhalten / Umsetzung
        int _qEMin_                  = data.getUnscaledValue("qEMin").intValue();
        int _korrekturfaktor_        = data.getUnscaledValue("Korrekturfaktor").intValue();
        int _bOpt_                   = data.getUnscaledValue("bOpt").intValue();
        int _berechnungsartBelegung_ = data.getUnscaledValue("BerechnungsartBelegung").intValue();
        int _mindesschaltzeit_       = data.getUnscaledValue("Mindesschaltzeit").intValue();
        int _nachlaufzeit_           = data.getUnscaledValue("Nachlaufzeit").intValue();
        int _maxErsteRotLaufZeit_    = data.getUnscaledValue("maxErsteRotLaufZeit").intValue();
        int _schwaechsteRotLaufZeit_ = data.getUnscaledValue("schwächsteRotLaufZeit").intValue();

        // Anzahl der nachfolgenden Programmblöcke auslesen...
        int _anzahlProgramme_ = data.getArray("PrgAlinea1").getLength();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(13 + (_anzahlProgramme_ * 5));

        // Fixen Teil des Telegramms in DE-Block übertragen...
        deApplicationData.writeIntAsLowHighByte(_qEMin_);
        deApplicationData.writeIntAsLowHighByte(_korrekturfaktor_);
        deApplicationData.writeIntAsByte(_bOpt_);
        deApplicationData.writeIntAsByte(_berechnungsartBelegung_);
        deApplicationData.writeIntAsByte(_mindesschaltzeit_);
        deApplicationData.writeIntAsByte(_nachlaufzeit_);
        deApplicationData.writeIntAsByte(_maxErsteRotLaufZeit_);
        deApplicationData.writeIntAsByte(_schwaechsteRotLaufZeit_);
        deApplicationData.writeIntAsByte(0);  // Reserve
        deApplicationData.writeIntAsByte(0);  // Reserve
        deApplicationData.writeIntAsByte(0);  // Reserve
        deApplicationData.writeIntAsByte(_anzahlProgramme_);

        // ...Schleife über die Programme direkt auslesen und in DE-Block schreiben
        for (int i = 0; i < _anzahlProgramme_; i++) {

            // Daten aus ATG auslesen...
            int _tEin_          = data.getArray("PrgAlinea1").getItem(i).getUnscaledValue("TEin").intValue();
            int _tAus_          = data.getArray("PrgAlinea1").getItem(i).getUnscaledValue("TAus").intValue();
            int _rotSchaltZeit_ = data.getArray("PrgAlinea1").getItem(i).getUnscaledValue("RotSchaltZeit").intValue();

            // ... und direkt in DE-Block eintragen
            deApplicationData.writeIntAsLowHighByte(_tEin_);
            deApplicationData.writeIntAsLowHighByte(_tAus_);
            deApplicationData.writeIntAsByte(_rotSchaltZeit_);
        }

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 33, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 9 : Typ: 33 Id: 35 Name: Betriebsparameter Alinea1 (Antwort). Standardverarbeitung wie bei Id 3,
     * {@link #Fg009Typ033Id003AtgTlsZfrBetriebsParameterAlinea1AspParameterIst(de.bsvrz.kex.tls.osi7.DeBlock, de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)},
     * aber als "nachgelieferte" Daten an den DaV übergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ033Id035AtgTlsZfrBetriebsParameterAlinea1AspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg009Typ033Id003AtgTlsZfrBetriebsParameterAlinea1AspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    // / Typ 34

    /**
     * FG 9 : Typ: 34 Id: 3 Name: Betriebsparameter Rotfahrerzähler (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ034Id003AtgTlsZfrBetriebsParameterRotFahrerZählerAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _erfassungsIntervallDauer_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _uebertragungsVerfahren_   = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("ErfassungsIntervallDauer").set(_erfassungsIntervallDauer_);
        data.getUnscaledValue("ÜbertragungsVerfahren").set(_uebertragungsVerfahren_);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 9 : Typ: 34 Id: 3 Name: Betriebsparameter Rotfahrerzähler (Senden). Beschreibung siehe [TLS 2011].
     *
     * @param result Der vom Datenverteiler empfangene Datensatz, der in einen De-Block konvertiert wird.
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg009Typ034Id003AtgTlsZfrBetriebsParameterRotFahrerZählerAspParameterSoll(ResultData result) {
        DeBlock deBlock = null;
        Data    data    = result.getData();  // Datensatz holen

        // Datenblock beseht aus folgenden Inhalten / Umsetzung
        int _erfassungsIntervallDauer_ = data.getUnscaledValue("ErfassungsIntervallDauer").intValue();
        int _uebertragungsVerfahren_   = data.getUnscaledValue("ÜbertragungsVerfahren").intValue();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(2);

        deApplicationData.writeIntAsByte(_erfassungsIntervallDauer_);
        deApplicationData.writeIntAsByte(_uebertragungsVerfahren_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 34, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 9 : Typ: 34 Id: 35 Name: Betriebsparameter Rotfahrerzähler (Antwort). Standardverarbeitung wie bei Id 3,
     * {@link #Fg009Typ034Id003AtgTlsZfrBetriebsParameterRotFahrerZählerAspParameterIst(de.bsvrz.kex.tls.osi7.DeBlock, de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)},
     * aber als "nachgelieferte" Daten an den DaV übergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ034Id035AtgTlsZfrBetriebsParameterRotFahrerZählerAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg009Typ034Id003AtgTlsZfrBetriebsParameterRotFahrerZählerAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    // / Typ 35

    /**
     * FG 9 : Typ: 35 Id: 3 Name: Betriebsparameter Alinea2 (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ035Id003AtgTlsZfrBetriebsParameterAlinea2AspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _vHfbStauEin_        = deBlock.getDeData().readUnsignedByteAsInt();
        int _tHfbVStauEin_       = deBlock.getDeData().readUnsignedByteAsInt();
        int _vHfbStauAus_        = deBlock.getDeData().readUnsignedByteAsInt();
        int _tHfbVStauAus_       = deBlock.getDeData().readUnsignedByteAsInt();
        int _bHfbStauEin_        = deBlock.getDeData().readUnsignedByteAsInt();
        int _tHfbBStauEin_       = deBlock.getDeData().readUnsignedByteAsInt();
        int _bHfbStauAus_        = deBlock.getDeData().readUnsignedByteAsInt();
        int _tHfbBStauAus_       = deBlock.getDeData().readUnsignedByteAsInt();
        int _bGelbAnSStauEin_    = deBlock.getDeData().readUnsignedByteAsInt();
        int _tGelbAnSBStauEin_   = deBlock.getDeData().readUnsignedByteAsInt();
        int _bGelbAnSStauAus_    = deBlock.getDeData().readUnsignedByteAsInt();
        int _tGelbAnSBStauAus_   = deBlock.getDeData().readUnsignedByteAsInt();
        int _bFGruenAnSStauEin_  = deBlock.getDeData().readUnsignedByteAsInt();
        int _tFGruenAnSBStauEin_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _bFGruenAnSStauAus_  = deBlock.getDeData().readUnsignedByteAsInt();
        int _tFGruenAnSBStauAus_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _bRStauSStauEin_     = deBlock.getDeData().readUnsignedByteAsInt();
        int _tRStauSBStauEin_    = deBlock.getDeData().readUnsignedByteAsInt();
        int _bRStauSStauAus_     = deBlock.getDeData().readUnsignedByteAsInt();
        int _tRStauSBStauAus_    = deBlock.getDeData().readUnsignedByteAsInt();
        int _bHfbMin_            = deBlock.getDeData().readUnsignedByteAsInt();
        int _tRest_              = deBlock.getDeData().readUnsignedByteAsInt();

        // 3 Reservebytes überlesen
        deBlock.getDeData().readUnsignedByteAsInt();
        deBlock.getDeData().readUnsignedByteAsInt();
        deBlock.getDeData().readUnsignedByteAsInt();

        // Fixen Teil des Telegramms in Ziel-ATG übertragen...
        data.getUnscaledValue("vHfbStauEin").set(_vHfbStauEin_);
        data.getUnscaledValue("tHfbVStauEin").set(_tHfbVStauEin_);
        data.getUnscaledValue("vHfbStauAus").set(_vHfbStauAus_);
        data.getUnscaledValue("tHfbVStauAus").set(_tHfbVStauAus_);
        data.getUnscaledValue("bHfbStauEin").set(_bHfbStauEin_);
        data.getUnscaledValue("tHfbBStauEin").set(_tHfbBStauEin_);
        data.getUnscaledValue("bHfbStauAus").set(_bHfbStauAus_);
        data.getUnscaledValue("tHfbBStauAus").set(_tHfbBStauAus_);
        data.getUnscaledValue("bGelbAnSStauEin").set(_bGelbAnSStauEin_);
        data.getUnscaledValue("tGelbAnSBStauEin").set(_tGelbAnSBStauEin_);
        data.getUnscaledValue("bGelbAnSStauAus").set(_bGelbAnSStauAus_);
        data.getUnscaledValue("tGelbAnSBStauAus").set(_tGelbAnSBStauAus_);
        data.getUnscaledValue("bFGrünAnSStauEin").set(_bFGruenAnSStauEin_);
        data.getUnscaledValue("tFGrünAnSBStauEin").set(_tFGruenAnSBStauEin_);
        data.getUnscaledValue("bFGrünAnSStauAus").set(_bFGruenAnSStauAus_);
        data.getUnscaledValue("tFGrünAnSBStauAus").set(_tFGruenAnSBStauAus_);
        data.getUnscaledValue("bRStauSStauEin").set(_bRStauSStauEin_);
        data.getUnscaledValue("tRStauSBStauEin").set(_tRStauSBStauEin_);
        data.getUnscaledValue("bRStauSStauAus").set(_bRStauSStauAus_);
        data.getUnscaledValue("tRStauSBStauAus").set(_tRStauSBStauAus_);
        data.getUnscaledValue("bHfbMin").set(_bHfbMin_);
        data.getUnscaledValue("tRest").set(_tRest_);

        // Anzahl der nachfolgenden Programmblöcke 1 auslesen...
        int _anzahlProgramme1_ = deBlock.getDeData().readUnsignedByteAsInt();

        // Datenfeld auf passende Länge initialisieren
        data.getArray("PrgRotZeitLockerungFGrünAnS").setLength(_anzahlProgramme1_);

        // ...Schleife über die Programme direkt auslesen und in ATG schreiben
        for (int i = 0; i < _anzahlProgramme1_; i++) {

            // Daten aus Telegramm einlesen...
            int _untereSchwelleRotSchaltZeit_ = deBlock.getDeData().readUnsignedByteAsInt();
            int _obereSchwelleRotSchaltZeit_  = deBlock.getDeData().readUnsignedByteAsInt();
            int _rotZeit_                     = deBlock.getDeData().readUnsignedByteAsInt();

            // ...und direkt in ATG / ATL PrgRotZeitLockerungFGrünAnS eintragen
            data.getArray("PrgRotZeitLockerungFGrünAnS").getItem(i).getUnscaledValue("untereSchwelleRotSchaltZeit").set(_untereSchwelleRotSchaltZeit_);
            data.getArray("PrgRotZeitLockerungFGrünAnS").getItem(i).getUnscaledValue("obereSchwelleRotSchaltZeit").set(_obereSchwelleRotSchaltZeit_);
            data.getArray("PrgRotZeitLockerungFGrünAnS").getItem(i).getUnscaledValue("RotZeit").set(_rotZeit_);
        }

        // Anzahl der nachfolgenden Programmblöcke 2 auslesen...
        int _anzahlProgramme2_ = deBlock.getDeData().readUnsignedByteAsInt();

        // Datenfeld auf passende Länge initialisieren
        data.getArray("PrgRotZeitLockerungRStauS").setLength(_anzahlProgramme2_);

        // ...Schleife über die Programme direkt auslesen und in ATG schreiben
        for (int i = 0; i < _anzahlProgramme2_; i++) {

            // Daten aus Telegramm einlesen...
            int _untereSchwelleRotSchaltZeit_ = deBlock.getDeData().readUnsignedByteAsInt();
            int _obereSchwelleRotSchaltZeit_  = deBlock.getDeData().readUnsignedByteAsInt();
            int _rotZeit_                     = deBlock.getDeData().readUnsignedByteAsInt();

            // ...und direkt in ATG / ATL PrgRotZeitLockerungRStauS eintragen
            data.getArray("PrgRotZeitLockerungRStauS").getItem(i).getUnscaledValue("untereSchwelleRotSchaltZeit").set(_untereSchwelleRotSchaltZeit_);
            data.getArray("PrgRotZeitLockerungRStauS").getItem(i).getUnscaledValue("obereSchwelleRotSchaltZeit").set(_obereSchwelleRotSchaltZeit_);
            data.getArray("PrgRotZeitLockerungRStauS").getItem(i).getUnscaledValue("RotZeit").set(_rotZeit_);
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 9 : Typ: 35 Id: 3 Name: Betriebsparameter Alinea2 (Senden). Beschreibung siehe [TLS 2011].
     *
     * @param result Der vom Datenverteiler empfangene Datensatz, der in einen De-Block konvertiert wird.
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg009Typ035Id003AtgTlsZfrBetriebsParameterAlinea2AspParameterSoll(ResultData result) {
        DeBlock deBlock = null;
        Data    data    = result.getData();  // Datensatz holen

        // Datenblock beseht aus folgenden Inhalten / Umsetzung
        int _vHfbStauEin_        = data.getUnscaledValue("vHfbStauEin").intValue();
        int _tHfbVStauEin_       = data.getUnscaledValue("tHfbVStauEin").intValue();
        int _vHfbStauAus_        = data.getUnscaledValue("vHfbStauAus").intValue();
        int _tHfbVStauAus_       = data.getUnscaledValue("tHfbVStauAus").intValue();
        int _bHfbStauEin_        = data.getUnscaledValue("bHfbStauEin").intValue();
        int _tHfbBStauEin_       = data.getUnscaledValue("tHfbBStauEin").intValue();
        int _bHfbStauAus_        = data.getUnscaledValue("bHfbStauAus").intValue();
        int _tHfbBStauAus_       = data.getUnscaledValue("tHfbBStauAus").intValue();
        int _bGelbAnSStauEin_    = data.getUnscaledValue("bGelbAnSStauEin").intValue();
        int _tGelbAnSBStauEin_   = data.getUnscaledValue("tGelbAnSBStauEin").intValue();
        int _bGelbAnSStauAus_    = data.getUnscaledValue("bGelbAnSStauAus").intValue();
        int _tGelbAnSBStauAus_   = data.getUnscaledValue("tGelbAnSBStauAus").intValue();
        int _bFGruenAnSStauEin_  = data.getUnscaledValue("bFGrünAnSStauEin").intValue();
        int _tFGruenAnSBStauEin_ = data.getUnscaledValue("tFGrünAnSBStauEin").intValue();
        int _bFGruenAnSStauAus_  = data.getUnscaledValue("bFGrünAnSStauAus").intValue();
        int _tFGruenAnSBStauAus_ = data.getUnscaledValue("tFGrünAnSBStauAus").intValue();
        int _bRStauSStauEin_     = data.getUnscaledValue("bRStauSStauEin").intValue();
        int _tRStauSBStauEin_    = data.getUnscaledValue("tRStauSBStauEin").intValue();
        int _bRStauSStauAus_     = data.getUnscaledValue("bRStauSStauAus").intValue();
        int _tRStauSBStauAus_    = data.getUnscaledValue("tRStauSBStauAus").intValue();
        int _bHfbMin_            = data.getUnscaledValue("bHfbMin").intValue();
        int _tRest_              = data.getUnscaledValue("tRest").intValue();

        // Anzahl der nachfolgenden Programmblöcke auslesen...
        int _anzahlProgramme1_ = data.getArray("PrgRotZeitLockerungFGrünAnS").getLength();
        int _anzahlProgramme2_ = data.getArray("PrgRotZeitLockerungRStauS").getLength();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(27 + (_anzahlProgramme1_ * 3) + (_anzahlProgramme2_ * 3));

        // Fixen Teil des Telegramms in DE-Block übertragen...
        deApplicationData.writeIntAsByte(_vHfbStauEin_);
        deApplicationData.writeIntAsByte(_tHfbVStauEin_);
        deApplicationData.writeIntAsByte(_vHfbStauAus_);
        deApplicationData.writeIntAsByte(_tHfbVStauAus_);
        deApplicationData.writeIntAsByte(_bHfbStauEin_);
        deApplicationData.writeIntAsByte(_tHfbBStauEin_);
        deApplicationData.writeIntAsByte(_bHfbStauAus_);
        deApplicationData.writeIntAsByte(_tHfbBStauAus_);
        deApplicationData.writeIntAsByte(_bGelbAnSStauEin_);
        deApplicationData.writeIntAsByte(_tGelbAnSBStauEin_);
        deApplicationData.writeIntAsByte(_bGelbAnSStauAus_);
        deApplicationData.writeIntAsByte(_tGelbAnSBStauAus_);
        deApplicationData.writeIntAsByte(_bFGruenAnSStauEin_);
        deApplicationData.writeIntAsByte(_tFGruenAnSBStauEin_);
        deApplicationData.writeIntAsByte(_bFGruenAnSStauAus_);
        deApplicationData.writeIntAsByte(_tFGruenAnSBStauAus_);
        deApplicationData.writeIntAsByte(_bRStauSStauEin_);
        deApplicationData.writeIntAsByte(_tRStauSBStauEin_);
        deApplicationData.writeIntAsByte(_bRStauSStauAus_);
        deApplicationData.writeIntAsByte(_tRStauSBStauAus_);
        deApplicationData.writeIntAsByte(_bHfbMin_);
        deApplicationData.writeIntAsByte(_tRest_);
        deApplicationData.writeIntAsByte(0);  // Reserve
        deApplicationData.writeIntAsByte(0);  // Reserve
        deApplicationData.writeIntAsByte(0);  // Reserve
        deApplicationData.writeIntAsByte(_anzahlProgramme1_);

        // ...Schleife über die Programme 1 direkt auslesen und in DE-Block schreiben
        for (int i = 0; i < _anzahlProgramme1_; i++) {

            // Daten aus ATG auslesen...
            int _untereSchwelleRotSchaltZeit_ = data.getArray("PrgRotZeitLockerungFGrünAnS").getItem(i).getUnscaledValue("untereSchwelleRotSchaltZeit").intValue();
            int _obereSchwelleRotSchaltZeit_  = data.getArray("PrgRotZeitLockerungFGrünAnS").getItem(i).getUnscaledValue("obereSchwelleRotSchaltZeit").intValue();
            int _rotZeit_                     = data.getArray("PrgRotZeitLockerungFGrünAnS").getItem(i).getUnscaledValue("RotZeit").intValue();

            // ... und direkt in DE-Block eintragen
            deApplicationData.writeIntAsByte(_untereSchwelleRotSchaltZeit_);
            deApplicationData.writeIntAsByte(_obereSchwelleRotSchaltZeit_);
            deApplicationData.writeIntAsByte(_rotZeit_);
        }

        deApplicationData.writeIntAsByte(_anzahlProgramme2_);

        // ...Schleife über die Programme 2 direkt auslesen und in DE-Block schreiben
        for (int i = 0; i < _anzahlProgramme2_; i++) {

            // Daten aus ATG auslesen...
            int _untereSchwelleRotSchaltZeit_ = data.getArray("PrgRotZeitLockerungRStauS").getItem(i).getUnscaledValue("untereSchwelleRotSchaltZeit").intValue();
            int _obereSchwelleRotSchaltZeit_  = data.getArray("PrgRotZeitLockerungRStauS").getItem(i).getUnscaledValue("obereSchwelleRotSchaltZeit").intValue();
            int _rotZeit_                     = data.getArray("PrgRotZeitLockerungRStauS").getItem(i).getUnscaledValue("RotZeit").intValue();

            // ... und direkt in DE-Block eintragen
            deApplicationData.writeIntAsByte(_untereSchwelleRotSchaltZeit_);
            deApplicationData.writeIntAsByte(_obereSchwelleRotSchaltZeit_);
            deApplicationData.writeIntAsByte(_rotZeit_);
        }

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 35, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 9 : Typ: 35 Id: 35 Name: Betriebsparameter Alinea2 (Antwort). Standardverarbeitung wie bei Id 3,
     * {@link #Fg009Typ035Id003AtgTlsZfrBetriebsParameterAlinea2AspParameterIst(de.bsvrz.kex.tls.osi7.DeBlock, de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)},
     * aber als "nachgelieferte" Daten an den DaV übergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ035Id035AtgTlsZfrBetriebsParameterAlinea2AspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg009Typ035Id003AtgTlsZfrBetriebsParameterAlinea2AspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 9 : Typ: 48 Id: 5 Name: Signalplan (Senden). Beschreibung siehe [TLS 2011].
     *
     * @param result Der vom Datenverteiler empfangene Datensatz, der in einen De-Block konvertiert wird.
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg009Typ048Id005AtgTlsZfrSignalPlanAspTlsVorgabe(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int _signalPlan_;
        int _anzahlZusaetzlicherKfz_;

        // Umsetzung
        Data data = result.getData();  // Datensatz holen

        _signalPlan_             = data.getUnscaledValue("SignalPlan").intValue();
        _anzahlZusaetzlicherKfz_ = data.getUnscaledValue("AnzahlZusätzlicherKfz").intValue();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(4);

        deApplicationData.writeIntAsByte(_signalPlan_);
        deApplicationData.writeIntAsByte(_anzahlZusaetzlicherKfz_);
        deApplicationData.writeIntAsByte(0);  // Reserve
        deApplicationData.writeIntAsByte(0);  // Reserve

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 48, 5, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 9 : Typ: 48 Id: 5 Name: Signalplan (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ048Id005AtgTlsZfrSignalPlanAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _signalPlan_             = deBlock.getDeData().readUnsignedByteAsInt();
        int _anzahlZusaetzlicherKfz_ = deBlock.getDeData().readUnsignedByteAsInt();

        // 2 Reserve-Bytes überlesen
        deBlock.getDeData().readUnsignedByteAsInt();
        deBlock.getDeData().readUnsignedByteAsInt();
        data.getUnscaledValue("SignalPlan").set(_signalPlan_);
        data.getUnscaledValue("AnzahlZusätzlicherKfz").set(_anzahlZusaetzlicherKfz_);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 9 : Typ: 48 Id: 37 Name: Signalplan (Antwort). Standardverarbeitung wie bei Id 5,
     * {@link #Fg009Typ048Id005AtgTlsZfrSignalPlanAspTlsZustand(de.bsvrz.kex.tls.osi7.DeBlock, de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)},
     * aber als "nachgelieferte" Daten an den DaV übergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ048Id037AtgTlsZfrSignalPlanAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 5, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg009Typ048Id005AtgTlsZfrSignalPlanAspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        } else {
            return resultData;
        }
    }

    /**
     * FG 9 : Typ: 49 Id: 5  Name: Helligkeit (Senden). Beschreibung siehe [TLS 2011].
     *
     * @param result Der vom Datenverteiler empfangene Datensatz, der in einen De-Block konvertiert wird.
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg009Typ049Id005AtgTlsZfrHelligkeitAspTlsVorgabe(ResultData result) {
        DeBlock deBlock = null;
        Data    data    = result.getData();  // Datensatz holen

        // Datenblock beseht aus folgenden Inhalten / Umsetzung
        int _Helligkeitswert_       = data.getUnscaledValue("HelligkeitsWert").intValue();
        int _Helligkeitsstatusbyte_ = data.getUnscaledValue("Status").intValue();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(2);

        deApplicationData.writeIntAsByte(_Helligkeitswert_);
        deApplicationData.writeIntAsByte(_Helligkeitsstatusbyte_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 49, 5, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 9 : Typ: 49 Id: 5  Name: Helligkeit (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ049Id005AtgTlsZfrHelligkeitAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Helligkeitswert aus Telegramm -------------------------------------------------------------------------------
        int _Helligkeitswert_ = deBlock.getDeData().readUnsignedByteAsInt();

        if ((_Helligkeitswert_ >= 0) && (_Helligkeitswert_ <= 100)) {
            data.getUnscaledValue("HelligkeitsWert").set(_Helligkeitswert_);
        } else {
            debug.warning("Helligkeitswert ungültig : " + _Helligkeitswert_ + Debug.NEWLINE + "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE + deBlock.toString());

            return resultData;
        }

        // Helligkeitswert aus Telegramm -------------------------------------------------------------------------------
        int _Helligkeitsstatusbyte_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Status").set(_Helligkeitsstatusbyte_ & 0x03);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 9 : Typ: 49 Id: 5  Name: Helligkeit (Antwort). Standardverarbeitung wie bei Id 5,
     * {@link #Fg009Typ049Id005AtgTlsZfrHelligkeitAspTlsZustand(de.bsvrz.kex.tls.osi7.DeBlock, de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)},
     * aber als "nachgelieferte" Daten an den DaV übergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ049Id037AtgTlsZfrHelligkeitAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 5, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg009Typ049Id005AtgTlsZfrHelligkeitAspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        } else {
            return resultData;
        }
    }

    /**
     * FG 9 : Typ: 65 Id: 4 Name: Rotfahrerzähler (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ065Id004AtgTlsZfrErgebnisMeldungRotFahrerZählerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Intervalllänge aus IntervalldatenHeader ---------------------------------------------------------------------
        data.getTimeValue("T").setSeconds(_eakConversionModul.getZfrIntervalDuration());

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _anzahlRotFahrer_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();

        data.getUnscaledValue("AnzahlRotFahrer").set(_anzahlRotFahrer_);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 9 : Typ: 65 Id: 36 Name: Rotfahrerzähler (Antwort). Standardverarbeitung wie bei Id 4,
     * {@link #Fg009Typ065Id004AtgTlsZfrErgebnisMeldungRotFahrerZählerAspTlsAntwort(de.bsvrz.kex.tls.osi7.DeBlock, de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)},
     * aber als "nachgelieferte" Daten an den DaV übergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ065Id036AtgTlsZfrErgebnisMeldungRotFahrerZählerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg009Typ065Id004AtgTlsZfrErgebnisMeldungRotFahrerZählerAspTlsAntwort(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        } else {
            return resultData;
        }
    }

    /**
     * FG 9 : Typ: 70 Id: 4 Name: ALINEA Schaltvorschläge (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ070Id004AtgTlsZfrAlineaSchaltVorschlägeAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _qZufluss_      = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _bHfb_          = deBlock.getDeData().readUnsignedByteAsInt();
        int _umlaufZeit_    = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _rotSchaltZeit_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("qZufluss").set(_qZufluss_);
        data.getUnscaledValue("bHfb").set(_bHfb_);
        data.getUnscaledValue("UmlaufZeit").set(_umlaufZeit_);
        data.getUnscaledValue("RotSchaltZeit").set(_rotSchaltZeit_);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 9 : Typ: 70 Id: 36 Name: ALINEA Schaltvorschläge (Antwort). Standardverarbeitung wie bei Id 4,
     * {@link #Fg009Typ070Id004AtgTlsZfrAlineaSchaltVorschlägeAspTlsAntwort(de.bsvrz.kex.tls.osi7.DeBlock, de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)},
     * aber als "nachgelieferte" Daten an den DaV übergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg009Typ070Id036AtgTlsZfrAlineaSchaltVorschlägeAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg009Typ070Id004AtgTlsZfrAlineaSchaltVorschlägeAspTlsAntwort(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        } else {
            return resultData;
        }
    }

    /**
     * FG 9 : Typ: 29 Id: 34 Name: Kanalsteuerung (Antwort). Standardverarbeitung wie bei ID 2,
     * {@link de.bsvrz.kex.tls.osi7.conversion.Fg009Default#Fg255Typ029Id002AtgTlsGloKanalSteuerungAspParameterIst(de.bsvrz.kex.tls.osi7.DeBlock,
     * de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)},
     * aber als "nachgelieferte" Daten an den DaV übergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg255Typ029Id034AtgTlsGloKanalSteuerungAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 2, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg255Typ029Id002AtgTlsGloKanalSteuerungAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 9 : Typ: 36 Id: 35 Name: Geographische Kenndaten (Antwort). Standardverarbeitung wie bei ID 3,
     * {@link de.bsvrz.kex.tls.osi7.conversion.Fg009Default#Fg255Typ036Id003AtgTlsGloGeographischeKenndatenAspTlsZustand(de.bsvrz.kex.tls.osi7.DeBlock,
     * de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)},
     * aber als "nachgelieferte" Daten an den DaV übergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg255Typ036Id035AtgTlsGloGeographischeKenndatenAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg255Typ036Id003AtgTlsGloGeographischeKenndatenAspTlsZustand(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 24.02.12
